package com.tengits.flutter_timpage_picker;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.text.TextUtils;
import android.widget.ImageView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;

import com.bumptech.glide.Glide;
import com.bumptech.glide.request.target.CustomTarget;
import com.bumptech.glide.request.transition.Transition;
import com.luck.picture.lib.basic.PictureSelectionCameraModel;
import com.luck.picture.lib.basic.PictureSelector;
import com.luck.picture.lib.config.PictureMimeType;
import com.luck.picture.lib.config.SelectMimeType;
import com.luck.picture.lib.engine.CompressEngine;
import com.luck.picture.lib.engine.CropEngine;
import com.luck.picture.lib.entity.LocalMedia;
import com.luck.picture.lib.interfaces.OnCallbackListener;
import com.luck.picture.lib.interfaces.OnResultCallbackListener;
import com.luck.picture.lib.utils.DateUtils;
import com.luck.picture.lib.utils.SdkVersionUtils;
import com.yalantis.ucrop.UCrop;
import com.yalantis.ucrop.UCropImageEngine;
import com.yalantis.ucrop.model.AspectRatio;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import top.zibin.luban.CompressionPredicate;
import top.zibin.luban.Luban;
import top.zibin.luban.OnCompressListener;
import top.zibin.luban.OnRenameListener;

/** FlutterTimpagePickerPlugin */
public class FlutterTimpagePickerPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Context context;
  private Activity mActivity;
  private List<LocalMedia> selectList = new ArrayList<>();

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "flutter_timpage_picker");
    channel.setMethodCallHandler(this);
    context = flutterPluginBinding.getApplicationContext();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if(call.method.equals("openCamera")) {
      openCamera(call, result);
    } else if(call.method.equals("showImagePicker")) {
      showImagePicker(call, result);
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  private void openCamera(@NonNull MethodCall call, @NonNull Result result) {
    boolean isCrop = call.argument("isCrop");
    boolean compress = call.argument("compress");
    PictureSelectionCameraModel cameraModel = PictureSelector.create(mActivity)
            .openCamera(SelectMimeType.ofImage())
            .setCropEngine(isCrop ? new ImageCropEngine(buildOptions(call)) : null)
            .setCompressEngine(compress ? new ImageCompressEngine() : null)
            .isOriginalControl(true)
            .setPermissionDescriptionListener(null);

    cameraModel.forResultActivity(new OnResultCallbackListener<LocalMedia>() {
      @Override
      public void onResult(ArrayList<LocalMedia> r) {
        ArrayList ret = new ArrayList();
        for(LocalMedia media : r) {
          String path = media.getPath();

          if (media.isCompressed() || media.isCut()) {
            path = media.getCompressPath();
          }

          if (media.isCut()) {
            path = media.getCutPath();
          }
          Map map = new HashMap();
          map.put("uri", "file://" + path);
          map.put("width", media.getWidth());
          map.put("height", media.getHeight());
          map.put("original_uri", "file://" + media.getPath());
          map.put("size", (int)new File(path).length());
          ret.add(map);
        }
        result.success(ret);
      }

      @Override
      public void onCancel() {
        result.success(null);
      }
    });
  }

  private void showImagePicker(@NonNull MethodCall call, @NonNull Result result) {
    int imageCount = call.argument("imageCount");
    boolean isCamera = call.argument("isCamera");
    boolean isCrop = call.argument("isCrop");
    boolean compress = call.argument("compress");
    boolean isOriginal = call.argument("allowPickingOriginalPhoto");
    boolean isGif = call.argument("isGif");
    boolean isRecordSelected = call.argument("isRecordSelected");

    int modeValue;
    if (imageCount == 1) {
      modeValue = 1;
    } else {
      modeValue = 2;
    }

    PictureSelector.create(mActivity)
            .openGallery(SelectMimeType.ofImage())
            .setImageEngine(GlideEngine.createGlideEngine())
            .setMaxSelectNum(imageCount)
            .setMinSelectNum(0)
            .setImageSpanCount(4)
            .setSelectionMode(modeValue)
            .isPreviewImage(true)
            .isOriginalControl(isOriginal)
            .isPreviewAudio(false)
            .isPreviewVideo(false)
            .isDisplayCamera(isCamera)
            .isSelectZoomAnim(true)
            .isGif(isGif)
            .setSelectedData(selectList)
            .setCropEngine(isCrop ? new ImageCropEngine(buildOptions(call)) : null)
            .setCompressEngine(compress ? new ImageCompressEngine() : null)
            .forResult(new OnResultCallbackListener<LocalMedia>() {
              @Override
              public void onResult(ArrayList<LocalMedia> r) {
                ArrayList ret = new ArrayList();
                if(isRecordSelected) {
                  selectList = r;
                }
                for(LocalMedia media : r) {

                  String path = media.getPath();

                  if (media.isCompressed() || media.isCut()) {
                    path = media.getCompressPath();
                  }

                  if (media.isCut()) {
                    path = media.getCutPath();
                  }
                  Map map = new HashMap();
                  map.put("uri", "file://" + path);
                  map.put("width", media.getWidth());
                  map.put("height", media.getHeight());
                  map.put("original_uri", "file://" + media.getPath());
                  map.put("size", (int)new File(path).length());
                  ret.add(map);
                }
                result.success(ret);
              }

              @Override
              public void onCancel() {
                  result.success(null);
              }
            });

  }

  private String getSandboxPath() {
    File externalFilesDir = context.getExternalFilesDir("");
    File customFile = new File(externalFilesDir.getAbsolutePath(), "Sandbox");
    if (!customFile.exists()) {
      customFile.mkdirs();
    }
    return customFile.getAbsolutePath() + File.separator;
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    mActivity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivity() {

  }


  private class ImageCropEngine implements CropEngine {

    private UCrop.Options options;

    ImageCropEngine(UCrop.Options ops) {
      this.options = ops;
    }

    @Override
    public void onStartCrop(Fragment fragment, LocalMedia currentLocalMedia,
                            ArrayList<LocalMedia> dataSource, int requestCode) {
      String currentCropPath = currentLocalMedia.getAvailablePath();
      Uri inputUri;
      if (PictureMimeType.isContent(currentCropPath) || PictureMimeType.isHasHttp(currentCropPath)) {
        inputUri = Uri.parse(currentCropPath);
      } else {
        inputUri = Uri.fromFile(new File(currentCropPath));
      }
      String fileName = DateUtils.getCreateFileName("CROP_") + ".jpg";
      Uri destinationUri = Uri.fromFile(new File(getSandboxPath(), fileName));
      UCrop.Options options = this.options;
      ArrayList<String> dataCropSource = new ArrayList<>();
      for (int i = 0; i < dataSource.size(); i++) {
        LocalMedia media = dataSource.get(i);
        dataCropSource.add(media.getAvailablePath());
      }
      UCrop uCrop = UCrop.of(inputUri, destinationUri, dataCropSource);
      //options.setMultipleCropAspectRatio(buildAspectRatios(dataSource.size()));
      uCrop.withOptions(options);
      uCrop.setImageEngine(new UCropImageEngine() {
        @Override
        public void loadImage(Context context, String url, ImageView imageView) {
          if (!ImageLoaderUtils.assertValidRequest(context)) {
            return;
          }
          Glide.with(context).load(url).override(180, 180).into(imageView);
        }

        @Override
        public void loadImage(Context context, Uri url, int maxWidth, int maxHeight, OnCallbackListener<Bitmap> call) {
          if (!ImageLoaderUtils.assertValidRequest(context)) {
            return;
          }
          Glide.with(context).asBitmap().override(maxWidth, maxHeight).load(url).into(new CustomTarget<Bitmap>() {
            @Override
            public void onResourceReady(@NonNull Bitmap resource, @Nullable Transition<? super Bitmap> transition) {
              if (call != null) {
                call.onCall(resource);
              }
            }

            @Override
            public void onLoadFailed(@Nullable Drawable errorDrawable) {
              if (call != null) {
                call.onCall(null);
              }
            }

            @Override
            public void onLoadCleared(@Nullable Drawable placeholder) {
            }
          });
        }
      });
      uCrop.start(fragment.getActivity(), fragment, requestCode);
    }
  }

  /**
   * 多图裁剪时每张对应的裁剪比例
   *
   * @param dataSourceCount
   * @return
   */
  private AspectRatio[] buildAspectRatios(int dataSourceCount) {
    AspectRatio[] aspectRatios = new AspectRatio[dataSourceCount];
    for (int i = 0; i < dataSourceCount; i++) {
      if (i == 0) {
        aspectRatios[i] = new AspectRatio("16:9", 16, 9);
      } else if (i == 1) {
        aspectRatios[i] = new AspectRatio("3:2", 3, 2);
      } else {
        aspectRatios[i] = new AspectRatio("原始比例", 0, 0);
      }
    }
    return aspectRatios;
  }

  private String[] getNotSupportCrop() {
    return new String[]{PictureMimeType.ofGIF(), PictureMimeType.ofWEBP()};
  }

  /**
   * 配制UCrop，可根据需求自我扩展
   *
   * @return
   */
  private UCrop.Options buildOptions(@NonNull MethodCall call) {
    int aspectX = call.argument("aspectX");
    int aspectY = call.argument("aspectY");
    boolean showCropCircle = call.argument("showCropCircle");
    boolean showCropFrame = call.argument("showCropFrame");
    boolean showCropGrid = call.argument("showCropGrid");
    UCrop.Options options = new UCrop.Options();
    options.setHideBottomControls(true);
    options.setFreeStyleCropEnabled(true);
    options.setShowCropFrame(showCropFrame);
    options.setShowCropGrid(showCropGrid);
    options.setCircleDimmedLayer(showCropCircle);
    options.withAspectRatio(aspectX, aspectY);
    options.setCropOutputPathDir(getSandboxPath());
    options.isCropDragSmoothToCenter(false);
    options.isUseCustomLoaderBitmap(false);
    options.setSkipCropMimeType(getNotSupportCrop());
    options.isForbidCropGifWebp(false);
    options.isForbidSkipMultipleCrop(false);
    options.setMaxScaleMultiplier(100);
    options.setStatusBarColor(ContextCompat.getColor(context, R.color.ps_color_grey));
    options.setToolbarColor(ContextCompat.getColor(context, R.color.ps_color_grey));
    options.setToolbarWidgetColor(ContextCompat.getColor(context, R.color.ps_color_white));

    return options;
  }

  private static class ImageCompressEngine implements CompressEngine {

    @Override
    public void onStartCompress(Context context, ArrayList<LocalMedia> list,
                                OnCallbackListener<ArrayList<LocalMedia>> listener) {
      // 自定义压缩
      List<Uri> compress = new ArrayList<>();
      for (int i = 0; i < list.size(); i++) {
        LocalMedia media = list.get(i);
        String availablePath = media.getAvailablePath();
        Uri uri = PictureMimeType.isContent(availablePath) || PictureMimeType.isHasHttp(availablePath)
                ? Uri.parse(availablePath)
                : Uri.fromFile(new File(availablePath));
        compress.add(uri);
      }
      if (compress.size() == 0) {
        listener.onCall(list);
        return;
      }
      Luban.with(context)
              .load(compress)
              .ignoreBy(100)
              .filter(new CompressionPredicate() {
                @Override
                public boolean apply(String path) {
                  return PictureMimeType.isUrlHasImage(path) && !PictureMimeType.isHasHttp(path);

                }
              })
              .setRenameListener(new OnRenameListener() {
                @Override
                public String rename(String filePath) {
                  int indexOf = filePath.lastIndexOf(".");
                  String postfix = indexOf != -1 ? filePath.substring(indexOf) : ".jpg";
                  return DateUtils.getCreateFileName("CMP_") + postfix;
                }
              })
              .setCompressListener(new OnCompressListener() {
                @Override
                public void onStart() {
                }

                @Override
                public void onSuccess(int index, File compressFile) {
                  LocalMedia media = list.get(index);
                  if (compressFile.exists() && !TextUtils.isEmpty(compressFile.getAbsolutePath())) {
                    media.setCompressed(true);
                    media.setCompressPath(compressFile.getAbsolutePath());
                    media.setSandboxPath(SdkVersionUtils.isQ() ? media.getCompressPath() : null);
                  }
                  if (index == list.size() - 1) {
                    listener.onCall(list);
                  }
                }

                @Override
                public void onError(int index, Throwable e) {
                  if (index != -1) {
                    LocalMedia media = list.get(index);
                    media.setCompressed(false);
                    media.setCompressPath(null);
                    media.setSandboxPath(null);
                    if (index == list.size() - 1) {
                      listener.onCall(list);
                    }
                  }
                }
              }).launch();
    }
  }
}
